/* debug.js -- Functions for debug events page

   Copyright 2013, 2014 by John K. Hinsdale.  See LICENSE.html for terms of use.
*/

// Flag: is display of captured events paused
GLOBAL__paused = false;

// Auto-refresh interval
GLOBAL__auto_refresh_millis = 1500;


// Update page contents (status, events table)
function update_page() {
    update_pause_button();
    update_status();
    update_events_table(true);
}

// Clear events
function clear_events() {
    b = chrome.extension.getBackgroundPage();
    b.debug_event_reset();
    update_page();
}

// Update the events table
function update_events_table(manual) {

    // Do nothing if an auto-update and we are paused
    if ( GLOBAL__paused && ! manual )
        return;
    
    // Do nothing if an auto-update and nothing has changed
    b = chrome.extension.getBackgroundPage();
    var dbg = b.GLOBAL__debug;
    if ( dbg.dirty || manual )
        update_events_table_rows();

    // Schedule next update
    if ( ! GLOBAL__paused )
        setTimeout(function() { update_events_table(false) }, GLOBAL__auto_refresh_millis);
}

// Actually do the update
function update_events_table_rows() {

    b = chrome.extension.getBackgroundPage();
    var dbg = b.GLOBAL__debug;

    // console.log("----------");
    // console.log("have " + dbg.fifo.length + " requests");
    // console.log(JSON.stringify(dbg.fifo));
    // console.log(JSON.stringify(dbg, null, 2));
    
    var table = document.getElementById("debug_events");
    var node = table.tBodies[0];
    // Wipe all rows
    while ( node.lastChild )
        node.removeChild(node.lastChild);
    // Add new rows
    var coloridx = 0;
    var colors = [ "#e0e0e0", "#d0d0d0" ];
    for (var i=0; i < dbg.fifo.length; i++) {
        var reqid = dbg.fifo[i];
        var inf = dbg.request[reqid];
        // Add row for request, with ID of request and rowspan to accomodate events
        var tr = document.createElement("tr");
        tr.style.backgroundColor = colors[coloridx];

        // console.log("inf.events.length: " + inf.events.length);

        // Cell for request ID, spanning request's event rows
        var td = document.createElement("td");
        td.rowSpan = inf.events.length + 1;
        td.innerHTML = reqid;
        tr.appendChild(td);
        
        // Cell for method/URL, spanning columns
        var rule_names = [];
        for (var r=0; r < inf.rules.length; r++)
            rule_names.push(inf.rules[r].name);
        var td = document.createElement("td");
        td.colSpan = 6; // Event, Rule, Action, Operation, Details, Details
        var html = "<b>" + inf.method + "</b> " + inf.url + "<br><b>Matched:</b> " + rule_names.join("<b> / </b>");
        shrink_font(td, html);
        td.innerHTML = html;
        tr.appendChild(td);

        // Add request-level row
        node.appendChild(tr);

        // Add rows for events in request
        for (var e=0; e < inf.events.length; e++) {
            var ev = inf.events[e];
            var tr = document.createElement("tr");
            var toadd = [ ev.event, ev.rule, ev.action, ev.operation, ev.arg1, ev.arg2 ];
            for (var t=0; t < toadd.length; t++) {
                var td = document.createElement("td");
                td.innerHTML = toadd[t];
                shrink_font(td, toadd[t]);
                tr.appendChild(td);
            }
            // console.log("appending event " + JSON.stringify(ev));
            node.appendChild(tr);
        }
        
        // Toggle request color
        coloridx = 1 - coloridx;
    }
}

// Pause/resume display of events
function pause_resume() {
    // Toggle pause status
    GLOBAL__paused = ! GLOBAL__paused;
    // If resumed, freshen page, else just the button text
    if ( ! GLOBAL__paused )
        update_page();
    else
        update_pause_button();
    update_status();
}

// Update pause button text
function update_pause_button() {
    // Update text of pause button
    var pause_button = document.getElementById("pause");
    if ( GLOBAL__paused )
        pause_button.innerHTML = "Resume";
    else
        pause_button.innerHTML = "Pause";
}    

// Update status text
function update_status() {
    b = chrome.extension.getBackgroundPage();
    var opts = b.GLOBAL__options;

    var txt = "Debug event capturing is <b>";
    if ( opts.debug ) {
        txt += "ON</b>.  Auto-refresh is <b>";
        if ( GLOBAL__paused )
            txt += "OFF</b> (click to resume).";
        else
            txt += "ON</b> (click to pause).";
    }
    else
        txt += "OFF</b>.";


    var status = document.getElementById("status");
    status.innerHTML = txt;
}

// Shrink font on element if large
function shrink_font(elt, s) {
    if ( (typeof s) != "string" )
        return;
    if ( s.length > 1000 )
        elt.style.fontSize = "60%";
    else if ( s.length > 500 )
        elt.style.fontSize = "75%";
    else if ( s.length > 100 )
        elt.style.fontSize = "90%";
}

////////   Init

// Pause/resume button
document.getElementById('pause').addEventListener('click', pause_resume);

// Refresh button
document.getElementById('refresh').addEventListener('click', function() { update_events_table(true) });

// Clear data button
document.getElementById('clear').addEventListener('click', function() { clear_events() });

// Click handler to cancel
document.getElementById('cancel').addEventListener('click', function() { window.close() });

// Update status and initial contents of events
update_page();

// $Id: debug.js,v 1.9 2014/12/23 12:38:42 hin Exp $
